#!/bin/bash

# 默认值
DEFAULT_BACKUP_DIR="/mnt"
DEFAULT_BACKUP_SPACE_GB=5

# 初始化变量
BACKUP_DIR=$DEFAULT_BACKUP_DIR
BACKUP_SPACE_GB=$DEFAULT_BACKUP_SPACE_GB

# 帮助信息
usage() {
    echo "Usage: $0 -a [backup|restore] -d BACKUP_DIR [-e EXCLUDE_DIRS] [-s BACKUP_SPACE_GB]"
    exit 1
}

# 解析选项
while getopts ":a:d:e:s:" opt; do
    case ${opt} in
        a )
            ACTION=$OPTARG
            ;;
        d )
            BACKUP_DIR=$OPTARG
            ;;
        e )
            EXCLUDE_DIRS=$OPTARG
            ;;
        s )
            BACKUP_SPACE_GB=$OPTARG
            ;;
        \? )
            usage
            ;;
    esac
done

# 检查必需的参数
if [ -z "$ACTION" ] || [ -z "$BACKUP_DIR" ]; then
    usage
fi

BACKUP_SPACE=$((BACKUP_SPACE_GB * 1024 * 1024))

# 检查目标目录是否存在
if [ ! -d "$BACKUP_DIR" ]; then
    echo "Backup directory $BACKUP_DIR does not exist."
    exit 1
fi

EXCLUDE_PARAMS=""
if [ -n "$EXCLUDE_DIRS" ]; then
    # 生成 rsync 的 --exclude 参数
    IFS=',' read -r -a EXCLUDE_ARRAY <<< "$EXCLUDE_DIRS"
    for EXCLUDE_DIR in "${EXCLUDE_ARRAY[@]}"; do
        EXCLUDE_PARAMS+="--exclude=$EXCLUDE_DIR "
    done
fi

EXCLUDE_PARAMS+="--exclude=$BACKUP_DIR "

# 备份功能
if [ "$ACTION" == "backup" ]; then

    # 检查目标目录的可用空间
    AVAILABLE_SPACE=$(df -k --output=avail "$BACKUP_DIR" | tail -n1)
    if [ "$AVAILABLE_SPACE" -lt "$BACKUP_SPACE" ]; then
        echo "Not enough space in $BACKUP_DIR. Required: $BACKUP_SPACE KB, Available: $AVAILABLE_SPACE KB."
        exit 1
    fi

    echo "Starting backup..."
    rsync -aAXHv --delete --exclude='/dev' --exclude='/proc' --exclude='/sys' --exclude='/tmp' --exclude='/run' --exclude='/lost+found' $EXCLUDE_PARAMS / "$BACKUP_DIR"
    echo "Backup completed."

# 恢复功能
elif [ "$ACTION" == "restore" ]; then
    echo "Starting restore..."
    rsync -aAXHv --delete --exclude='/dev' --exclude='/proc' --exclude='/sys' --exclude='/tmp' --exclude='/run' --exclude='/lost+found' $EXCLUDE_PARAMS "$BACKUP_DIR" /
    echo "Restore completed."

# 无效的操作
else
    echo "Invalid action: $ACTION. Use 'backup' or 'restore'."
    exit 1
fi